@@ -1,7 +1,14 @@
 # -*- coding: utf-8 -*-
 #
 # Picard, the next-generation MusicBrainz tagger
+#
 # Copyright (C) 2014 Lukáš Lalinský
+# Copyright (C) 2015 Sophist-UK
+# Copyright (C) 2015 Wieland Hoffmann
+# Copyright (C) 2015, 2018, 2021 Philipp Wolfer
+# Copyright (C) 2016-2017 Sambhav Kothari
+# Copyright (C) 2017 Frederik “Freso” S. Olesen
+# Copyright (C) 2018-2022 Laurent Monin
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -17,19 +24,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
+
 from functools import partial
+from json.decoder import JSONDecodeError
 import time
+import urllib.parse
 
-from PyQt5.QtCore import (
-    QUrl,
-    QUrlQuery,
-)
-from PyQt5.QtNetwork import QNetworkRequest
-
-from picard import (
-    config,
-    log,
-)
+from picard import log
+from picard.config import get_config
 from picard.const import (
     MUSICBRAINZ_OAUTH_CLIENT_ID,
     MUSICBRAINZ_OAUTH_CLIENT_SECRET,
@@ -44,8 +46,16 @@ class OAuthManager(object):
 
     def __init__(self, webservice):
         self.webservice = webservice
-        self.setting = config.setting
-        self.persist = config.persist
+
+    @property
+    def setting(self):
+        config = get_config()
+        return config.setting
+
+    @property
+    def persist(self):
+        config = get_config()
+        return config.persist
 
     @property
     def host(self):
@@ -140,13 +150,20 @@ class OAuthManager(object):
                 self.forget_access_token()
                 self.refresh_access_token(callback)
 
+    def url(self, path=None, params=None):
+        return build_qurl(
+            self.host, self.port, path=path,
+            queryargs=params
+        )
+
     def get_authorization_url(self, scopes):
-        params = {"response_type": "code", "client_id":
-                  MUSICBRAINZ_OAUTH_CLIENT_ID, "redirect_uri":
-                  "urn:ietf:wg:oauth:2.0:oob", "scope": scopes}
-        url = build_qurl(self.host, self.port, path="/oauth2/authorize",
-                         queryargs=params)
-        return bytes(url.toEncoded()).decode()
+        params = {
+            "response_type": "code",
+            "client_id": MUSICBRAINZ_OAUTH_CLIENT_ID,
+            "redirect_uri": "urn:ietf:wg:oauth:2.0:oob",
+            "scope": scopes,
+        }
+        return bytes(self.url(path="/oauth2/authorize", params=params).toEncoded()).decode()
 
     def set_refresh_token(self, refresh_token, scopes):
         log.debug("OAuth: got refresh_token %s with scopes %s", refresh_token, scopes)
@@ -158,28 +175,34 @@ class OAuthManager(object):
         self.access_token = access_token
         self.access_token_expires = int(time.time() + expires_in - 60)
 
+    @staticmethod
+    def _query_data(params):
+        return urllib.parse.urlencode({key: value for key, value in params.items() if key})
+
     def refresh_access_token(self, callback):
         log.debug("OAuth: refreshing access_token with a refresh_token %s", self.refresh_token)
-        path = "/oauth2/token"
-        url = QUrl()
-        url_query = QUrlQuery()
-        url_query.addQueryItem("grant_type", "refresh_token")
-        url_query.addQueryItem("refresh_token", self.refresh_token)
-        url_query.addQueryItem("client_id", MUSICBRAINZ_OAUTH_CLIENT_ID)
-        url_query.addQueryItem("client_secret", MUSICBRAINZ_OAUTH_CLIENT_SECRET)
-        url.setQuery(url_query.query(QUrl.FullyEncoded))
-        data = url.query()
-        self.webservice.post(self.host, self.port, path, data,
-                             partial(self.on_refresh_access_token_finished, callback),
-                             mblogin=True, priority=True, important=True,
-                             request_mimetype="application/x-www-form-urlencoded")
+        params = {
+            "grant_type": "refresh_token",
+            "refresh_token": self.refresh_token,
+            "client_id": MUSICBRAINZ_OAUTH_CLIENT_ID,
+            "client_secret": MUSICBRAINZ_OAUTH_CLIENT_SECRET,
+        }
+        self.webservice.post_url(
+            url=self.url(path="/oauth2/token"),
+            data=self._query_data(params),
+            handler=partial(self.on_refresh_access_token_finished, callback),
+            mblogin=True,
+            priority=True,
+            important=True,
+            request_mimetype="application/x-www-form-urlencoded",
+        )
 
     def on_refresh_access_token_finished(self, callback, data, http, error):
         access_token = None
         try:
             if error:
                 log.error("OAuth: access_token refresh failed: %s", data)
-                if http.attribute(QNetworkRequest.HttpStatusCodeAttribute) == 400:
+                if self._http_code(http) == 400:
                     response = load_json(data)
                     if response["error"] == "invalid_grant":
                         self.forget_refresh_token()
@@ -193,52 +216,73 @@ class OAuthManager(object):
 
     def exchange_authorization_code(self, authorization_code, scopes, callback):
         log.debug("OAuth: exchanging authorization_code %s for an access_token", authorization_code)
-        path = "/oauth2/token"
-        url = QUrl()
-        url_query = QUrlQuery()
-        url_query.addQueryItem("grant_type", "authorization_code")
-        url_query.addQueryItem("code", authorization_code)
-        url_query.addQueryItem("client_id", MUSICBRAINZ_OAUTH_CLIENT_ID)
-        url_query.addQueryItem("client_secret", MUSICBRAINZ_OAUTH_CLIENT_SECRET)
-        url_query.addQueryItem("redirect_uri", "urn:ietf:wg:oauth:2.0:oob")
-        url.setQuery(url_query.query(QUrl.FullyEncoded))
-        data = url.query()
-        self.webservice.post(self.host, self.port, path, data,
-                             partial(self.on_exchange_authorization_code_finished, scopes, callback),
-                             mblogin=True, priority=True, important=True,
-                             request_mimetype="application/x-www-form-urlencoded")
+        params = {
+            "grant_type": "authorization_code",
+            "code": authorization_code,
+            "client_id": MUSICBRAINZ_OAUTH_CLIENT_ID,
+            "client_secret": MUSICBRAINZ_OAUTH_CLIENT_SECRET,
+            "redirect_uri": "urn:ietf:wg:oauth:2.0:oob",
+        }
+        self.webservice.post_url(
+            url=self.url(path="/oauth2/token"),
+            data=self._query_data(params),
+            handler=partial(self.on_exchange_authorization_code_finished, scopes, callback),
+            mblogin=True,
+            priority=True,
+            important=True,
+            request_mimetype="application/x-www-form-urlencoded",
+        )
 
     def on_exchange_authorization_code_finished(self, scopes, callback, data, http, error):
         successful = False
+        error_msg = None
         try:
             if error:
                 log.error("OAuth: authorization_code exchange failed: %s", data)
+                error_msg = self._extract_error_description(http, data)
             else:
                 self.set_refresh_token(data["refresh_token"], scopes)
                 self.set_access_token(data["access_token"], data["expires_in"])
                 successful = True
         except Exception as e:
             log.error('OAuth: Unexpected error handling authorization code response: %r', e)
+            error_msg = _('Unexpected authentication error')
         finally:
-            callback(successful=successful)
+            callback(successful=successful, error_msg=error_msg)
 
     def fetch_username(self, callback):
         log.debug("OAuth: fetching username")
-        path = "/oauth2/userinfo"
-        self.webservice.get(self.host, self.port, path,
-                            partial(self.on_fetch_username_finished, callback),
-                            mblogin=True, priority=True, important=True)
+        self.webservice.get_url(
+            url=self.url(path="/oauth2/userinfo"),
+            handler=partial(self.on_fetch_username_finished, callback),
+            mblogin=True,
+            priority=True,
+            important=True,
+        )
 
     def on_fetch_username_finished(self, callback, data, http, error):
         successful = False
+        error_msg = None
         try:
             if error:
                 log.error("OAuth: username fetching failed: %s", data)
+                error_msg = self._extract_error_description(http, data)
             else:
                 self.username = data["sub"]
                 log.debug("OAuth: got username %s", self.username)
                 successful = True
         except Exception as e:
             log.error('OAuth: Unexpected error handling username fetch response: %r', e)
+            error_msg = _('Unexpected authentication error')
         finally:
-            callback(successful)
+            callback(successful=successful, error_msg=error_msg)
+
+    def _http_code(self, http):
+        return self.webservice.http_response_code(http)
+
+    def _extract_error_description(self, http, data):
+        try:
+            response = load_json(data)
+            return response['error_description']
+        except (JSONDecodeError, KeyError, TypeError):
+            return _('Unexpected request error (HTTP code %s)') % self._http_code(http)
